@@ -1,48 +1,14 @@
-/****************************************************************************
-**
-** Copyright (C) 2012 Denis Shienkov <denis.shienkov@gmail.com>
-** Copyright (C) 2012 Laszlo Papp <lpapp@kde.org>
-** Copyright (C) 2012 Andre Hartmann <aha_1980@gmx.de>
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtSerialPort module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2012 Denis Shienkov <denis.shienkov@gmail.com>
+// Copyright (C) 2012 Laszlo Papp <lpapp@kde.org>
+// Copyright (C) 2012 Andre Hartmann <aha_1980@gmx.de>
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qserialport_p.h"
 #include "qserialportinfo_p.h"
 
+#include <QtCore/qdeadlinetimer.h>
 #include <QtCore/qelapsedtimer.h>
+#include <QtCore/qloggingcategory.h>
 #include <QtCore/qmap.h>
 #include <QtCore/qsocketnotifier.h>
 #include <QtCore/qstandardpaths.h>
@@ -55,7 +21,7 @@
 #include <sys/time.h>
 #include <unistd.h>
 
-#ifdef Q_OS_OSX
+#ifdef Q_OS_MACOS
 #if defined(MAC_OS_X_VERSION_10_4) && (MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_4)
 #include <IOKit/serial/ioss.h>
 #endif
@@ -70,10 +36,10 @@
 # ifdef Q_OS_ANDROID
 #  include <android/api-level.h>
 # else
-#  define __ANDROID_API__ 21
+#  define __ANDROID_API__ 16
 # endif
 
-# if !defined(Q_OS_ANDROID) || (!defined(Q_PROCESSOR_X86) && __ANDROID_API__ < 21)
+# if !defined(Q_OS_ANDROID) || (!defined(Q_PROCESSOR_X86) && __ANDROID_API__ < 16)
 struct termios2 {
     tcflag_t c_iflag;       /* input mode flags */
     tcflag_t c_oflag;       /* output mode flags */
@@ -100,6 +66,9 @@ struct termios2 {
 
 #endif
 
+[[maybe_unused]]
+Q_STATIC_LOGGING_CATEGORY(lcUnixWarnings, "qt.serialport.unix.warnings");
+
 QT_BEGIN_NAMESPACE
 
 QString serialPortLockFilePath(const QString &portName)
@@ -156,7 +125,7 @@ public:
     }
 
 protected:
-    bool event(QEvent *e) Q_DECL_OVERRIDE
+    bool event(QEvent *e) override
     {
         if (e->type() == QEvent::SockAct) {
             dptr->readNotification();
@@ -179,7 +148,7 @@ public:
     }
 
 protected:
-    bool event(QEvent *e) Q_DECL_OVERRIDE
+    bool event(QEvent *e) override
     {
         if (e->type() == QEvent::SockAct) {
             dptr->completeAsyncWrite();
@@ -262,6 +231,11 @@ static inline void qt_set_parity(termios *tio, QSerialPort::Parity parity)
         tio->c_cflag |= PARENB | PARODD;
         break;
     default:
+#ifndef CMSPAR
+    case QSerialPort::SpaceParity:
+    case QSerialPort::MarkParity:
+        qCWarning(lcUnixWarnings, "Space and Mark parity is not supported by the OS.");
+#endif
         tio->c_cflag |= PARENB;
         tio->c_iflag |= PARMRK | INPCK;
         tio->c_iflag &= ~IGNPAR;
@@ -316,7 +290,7 @@ bool QSerialPortPrivate::open(QIODevice::OpenMode mode)
         return false;
     }
 
-    QScopedPointer<QLockFile> newLockFileScopedPointer(new QLockFile(lockFilePath));
+    auto newLockFileScopedPointer = std::make_unique<QLockFile>(lockFilePath);
 
     if (!newLockFileScopedPointer->tryLock()) {
         setError(QSerialPortErrorInfo(QSerialPort::PermissionError, QSerialPort::tr("Permission error while locking the device")));
@@ -349,7 +323,7 @@ bool QSerialPortPrivate::open(QIODevice::OpenMode mode)
         return false;
     }
 
-    lockFileScopedPointer.swap(newLockFileScopedPointer);
+    lockFileScopedPointer = std::move(newLockFileScopedPointer);
 
     return true;
 }
@@ -526,7 +500,8 @@ bool QSerialPortPrivate::waitForBytesWritten(int msecs)
     for (;;) {
         bool readyToRead = false;
         bool readyToWrite = false;
-        if (!waitForReadOrWrite(&readyToRead, &readyToWrite, true, !writeBuffer.isEmpty(),
+        const bool checkRead = q_func()->isReadable();
+        if (!waitForReadOrWrite(&readyToRead, &readyToWrite, checkRead, !writeBuffer.isEmpty(),
                                 qt_subtract_from_timeout(msecs, stopWatch.elapsed()))) {
             return false;
         }
@@ -649,7 +624,7 @@ bool QSerialPortPrivate::setCustomBaudRate(qint32 baudRate, QSerialPort::Directi
     return setStandardBaudRate(B38400, directions);
 }
 
-#elif defined(Q_OS_OSX)
+#elif defined(Q_OS_MACOS)
 
 bool QSerialPortPrivate::setCustomBaudRate(qint32 baudRate, QSerialPort::Directions directions)
 {
@@ -785,7 +760,7 @@ bool QSerialPortPrivate::readNotification()
 
     buffer.chop(bytesToRead - qMax(readBytes, qint64(0)));
 
-    if (readBytes <= 0) {
+    if (readBytes < 0) {
         QSerialPortErrorInfo error = getSystemError();
         if (error.errorCode != QSerialPort::ResourceError)
             error.errorCode = QSerialPort::ReadError;
@@ -793,6 +768,8 @@ bool QSerialPortPrivate::readNotification()
             setReadNotificationEnabled(false);
         setError(error);
         return false;
+    } else if (readBytes == 0) {
+        return false;
     }
 
     newBytes = buffer.size() - newBytes;
@@ -869,6 +846,11 @@ inline bool QSerialPortPrivate::initialize(QIODevice::OpenMode mode)
 
     restoredTermios = tio;
 
+    auto restoreParametersOnError = qScopeGuard([this] {
+        if (settingsRestoredOnClose)
+            ::tcsetattr(descriptor, TCSANOW, &restoredTermios);
+    });
+
     qt_set_common_props(&tio, mode);
     qt_set_databits(&tio, dataBits);
     qt_set_parity(&tio, parity);
@@ -884,6 +866,10 @@ inline bool QSerialPortPrivate::initialize(QIODevice::OpenMode mode)
     if (mode & QIODevice::ReadOnly)
         setReadNotificationEnabled(true);
 
+    // flush IO buffers
+    clear(QSerialPort::AllDirections);
+
+    restoreParametersOnError.dismiss();
     return true;
 }
 
@@ -946,7 +932,7 @@ QSerialPortErrorInfo QSerialPortPrivate::getSystemError(int systemErrorCode) con
     case EBADF:
         error.errorCode = QSerialPort::ResourceError;
         break;
-#ifdef Q_OS_OSX
+#ifdef Q_OS_MACOS
     case ENXIO:
         error.errorCode = QSerialPort::ResourceError;
         break;
@@ -1027,7 +1013,7 @@ bool QSerialPortPrivate::waitForReadOrWrite(bool *selectForRead, bool *selectFor
     if (checkWrite)
         pfd.events |= POLLOUT;
 
-    const int ret = qt_poll_msecs(&pfd, 1, msecs);
+    const int ret = qt_safe_poll(&pfd, 1, QDeadlineTimer(msecs));
     if (ret < 0) {
         setError(getSystemError());
         return false;
@@ -1094,7 +1080,11 @@ qint64 QSerialPortPrivate::writePerChar(const char *data, qint64 maxSize)
         par ^= parity == QSerialPort::MarkParity;
         if (par ^ (tio.c_cflag & PARODD)) { // Need switch parity mode?
             tio.c_cflag ^= PARODD;
-            flush(); //force sending already buffered data, because setTermios(&tio); cleares buffers
+            //force sending already buffered data, because setTermios(&tio); cleares buffers
+            if (::tcdrain(descriptor) == -1) {
+                setError(getSystemError());
+                break;
+            }
             //todo: add receiving buffered data!!!
             if (!setTermios(&tio))
                 break;
